@@ -42,12 +42,7 @@
 #include <linux/pci.h>
 
 #include "compiler.h"
-#include "460gxPCI.h"
-#include "e8870PCI.h"
-#include "zx1PCI.h"
-#include "altixPCI.h"
 #include "Pci.h"
-#include "ia64Pci.h"
 
 /*
  * We use special in/out routines here since Altix platforms require the
@@ -65,7 +60,7 @@ static int ia64_port_to_fd(unsigned long port)
     return (port >> 24) & 0xffffffff;
 }
 
-_X_EXPORT void outb(unsigned long port, unsigned char val)
+void outb(unsigned long port, unsigned char val)
 {
     int fd = ia64_port_to_fd(port);
 
@@ -85,7 +80,7 @@ _X_EXPORT void outb(unsigned long port, unsigned char val)
     return;
 }
 
-_X_EXPORT void outw(unsigned long port, unsigned short val)
+void outw(unsigned long port, unsigned short val)
 {
     int fd = ia64_port_to_fd(port);
 
@@ -105,7 +100,7 @@ _X_EXPORT void outw(unsigned long port, unsigned short val)
     return;
 }
 
-_X_EXPORT void outl(unsigned long port, unsigned int val)
+void outl(unsigned long port, unsigned int val)
 {
     int fd = ia64_port_to_fd(port);
 
@@ -125,7 +120,7 @@ _X_EXPORT void outl(unsigned long port, unsigned int val)
     return;
 }
 
-_X_EXPORT unsigned int inb(unsigned long port)
+unsigned int inb(unsigned long port)
 {
     int fd = ia64_port_to_fd(port);
     unsigned char val;
@@ -147,7 +142,7 @@ _X_EXPORT unsigned int inb(unsigned long port)
     return val;
 }
 
-_X_EXPORT unsigned int inw(unsigned long port)
+unsigned int inw(unsigned long port)
 {
     int fd = ia64_port_to_fd(port);
     unsigned short val;
@@ -169,7 +164,7 @@ _X_EXPORT unsigned int inw(unsigned long port)
     return val;
 }
 
-_X_EXPORT unsigned int inl(unsigned long port)
+unsigned int inl(unsigned long port)
 {
     int fd = ia64_port_to_fd(port);
     unsigned int val;
@@ -191,53 +186,3 @@ _X_EXPORT unsigned int inl(unsigned long port)
     return val;
 }
 
-void
-ia64ScanPCIWrapper(scanpciWrapperOpt flags)
-{
-    static IA64Chipset chipset = NONE_CHIPSET;
-    
-    if (flags == SCANPCI_INIT) {
-
-	/* PCI configuration space probes should be done first */
-	if (xorgProbe460GX(flags)) {
-	    chipset = I460GX_CHIPSET;
-	    xf86PreScan460GX();	
-	    return;
-	} else if (xorgProbeE8870(flags)) {
-	    chipset = E8870_CHIPSET;
-	    xf86PreScanE8870();
-	    return;
-	}
-#ifdef OS_PROBE_PCI_CHIPSET
-	chipset = OS_PROBE_PCI_CHIPSET(flags);
-	switch (chipset) {
-	    case ZX1_CHIPSET:
-		xf86PreScanZX1();
-		return;
-	    case ALTIX_CHIPSET:
-		xf86PreScanAltix();
-		return;
-	    default:
-		return;
-	}
-#endif
-    } else /* if (flags == SCANPCI_TERM) */ {
-
-	switch (chipset) {
-	    case I460GX_CHIPSET:
-		xf86PostScan460GX();
-		return;
-	    case E8870_CHIPSET:
-		xf86PostScanE8870();
-		return;
-	    case ZX1_CHIPSET:
-		xf86PostScanZX1();
-		return;
-	    case ALTIX_CHIPSET:
-		xf86PostScanAltix();
-		return;
-	    default:
-		return;
-	}
-    }
-}
